@@ -5,11 +5,7 @@ import json
 import dill
 from figaro.exceptions import FIGAROException
 from figaro.mixture import mixture
-try:
-    from figaro.cosmology import CosmologicalParameters
-    lal_flag = True
-except ModuleNotFoundError:
-    lal_flag = False
+from figaro.cosmology import CosmologicalParameters
 from pathlib import Path
 from scipy.optimize import newton
 from tqdm import tqdm
@@ -22,9 +18,11 @@ GW_par = {'m1'                 : 'mass_1_source',
           'm1_detect'          : 'mass_1',
           'm2_detect'          : 'mass_2',
           'mc'                 : 'chirp_mass_source',
+          'mc_detect'          : 'chirp_mass',
           'mt'                 : 'total_mass_source',
           'z'                  : 'redshift',
           'q'                  : 'mass_ratio',
+          'eta'                : 'symmetric_mass_ratio',
           'chi_eff'            : 'chi_eff',
           'ra'                 : 'ra',
           'dec'                : 'dec',
@@ -32,12 +30,22 @@ GW_par = {'m1'                 : 'mass_1_source',
           'cos_theta_jn'       : 'cos_theta_jn',
           'cos_tilt_1'         : 'cos_tilt_1',
           'cos_tilt_2'         : 'cos_tilt_2',
+          's1x'                : 'spin_1x',
+          's2x'                : 'spin_2x',
+          's1y'                : 'spin_1y',
+          's2y'                : 'spin_2y',
           's1z'                : 'spin_1z',
           's2z'                : 'spin_2z',
           's1'                 : 'spin_1',
           's2'                 : 'spin_2',
+          'psi'                : 'psi',
+          'cos_iota'           : 'cos_iota',
+          'phase'              : 'phase',
+          'tc'                 : 'geocent_time',
           'snr'                : 'network_matched_filter_snr',
           'far'                : 'far',
+          'log_prior'          : 'log_prior',
+          'log_likelihood'     : 'log_likelihood',
           }
 
 def _find_redshift(omega, dl):
@@ -45,11 +53,11 @@ def _find_redshift(omega, dl):
     Find redshift given a luminosity distance and a cosmology using Newton's method
     
     Arguments:
-        :CosmologicalParameters omega: cosmology (see cosmology.pyx for definition)
-        :double dl:                    luminosity distance
+        CosmologicalParameters omega: cosmology (see cosmology.pyx for definition)
+        double dl:                    luminosity distance
     
     Returns:
-        :double: redshift
+        double: redshift
     """
     def objective(z, omega, dl):
         return dl - omega.LuminosityDistance_double(z)
@@ -68,21 +76,21 @@ def load_single_event(event, seed = False, par = None, n_samples = -1, h = 0.674
     Not all GW parameters are implemented: run figaro.load.available_gw_pars() for a list of the available parameters.
     
     Arguments:
-        :str or Path file: file with samples
-        :bool seed:        fixes the seed to a default value (1) for reproducibility
-        :list-of-str par:  list with parameter(s) to extract from GW posteriors (m1, m2, mc, z, chi_effective)
-        :int n_samples:    number of samples for (random) downsampling. Default -1: all samples
-        :double h:         Hubble constant H0/100 [km/(s*Mpc)]
-        :double om:        matter density parameter
-        :double ol:        cosmological constant density parameter
-        :bool volume:      if True, loads RA, dec and Luminosity distance (for skymaps)
-        :str waveform:     waveform family to be used ('combined', 'seob', 'imr')
-        :double snr_threhsold: SNR threshold for event filtering. For injection analysis only.
-        :double far_threshold: FAR threshold for event filtering. For injection analysis only.
+        str or Path file: file with samples
+        bool seed:        fixes the seed to a default value (1) for reproducibility
+        list-of-str par:  list with parameter(s) to extract from GW posteriors (m1, m2, mc, z, chi_effective)
+        int n_samples:    number of samples for (random) downsampling. Default -1: all samples
+        double h:         Hubble constant H0/100 [km/(s*Mpc)]
+        double om:        matter density parameter
+        double ol:        cosmological constant density parameter
+        bool volume:      if True, loads RA, dec and Luminosity distance (for skymaps)
+        str waveform:     waveform family to be used ('combined', 'seob', 'imr')
+        double snr_threhsold: SNR threshold for event filtering. For injection analysis only.
+        double far_threshold: FAR threshold for event filtering. For injection analysis only.
         
     Returns:
-        :np.ndarray: samples
-        :np.ndarray: name
+        np.ndarray: samples
+        np.ndarray: name
     '''
     if seed:
         rdstate = np.random.RandomState(seed = 1)
@@ -98,11 +106,11 @@ def load_single_event(event, seed = False, par = None, n_samples = -1, h = 0.674
         if par is not None:
             warnings.warn("Par names (or volume keyword) are ignored for .txt/.dat/.csv files")
         if n_samples > -1:
-            samples = np.atleast_1d(np.genfromtxt(event))
+            samples = np.atleast_1d(np.loadtxt(event))
             s = int(min([n_samples, len(samples)]))
             out = samples[rdstate.choice(np.arange(len(samples)), size = s, replace = False)]
         else:
-            out = np.genfromtxt(event)
+            out = np.loadtxt(event)
     else:
         # Check that a list of parameters is passed
         if par is None:
@@ -132,20 +140,20 @@ def load_data(path, seed = False, par = None, n_samples = -1, h = 0.674, om = 0.
     Not all GW parameters are implemented: run figaro.load.available_gw_pars() for a list of available parameters.
     
     Arguments:
-        :str or Path path: folder with data files
-        :bool seed:        fixes the seed to a default value (1) for reproducibility
-        :list-of-str par:  list with parameter(s) to extract from GW posteriors
-        :int n_samples:    number of samples for (random) downsampling. Default -1: all samples
-        :double h:         Hubble constant H0/100 [km/(s*Mpc)]
-        :double om:        matter density parameter
-        :double ol:        cosmological constant density parameter
-        :str waveform:     waveform family to be used ('combined', 'seob', 'imr')
-        :double snr_threhsold: SNR threshold for event filtering. For injection analysis only.
-        :double far_threshold: FAR threshold for event filtering. For injection analysis only.
+        str or Path path: folder with data files
+        bool seed:        fixes the seed to a default value (1) for reproducibility
+        list-of-str par:  list with parameter(s) to extract from GW posteriors
+        int n_samples:    number of samples for (random) downsampling. Default -1: all samples
+        double h:         Hubble constant H0/100 [km/(s*Mpc)]
+        double om:        matter density parameter
+        double ol:        cosmological constant density parameter
+        str waveform:     waveform family to be used ('combined', 'seob', 'imr')
+        double snr_threhsold: SNR threshold for event filtering. For injection analysis only.
+        double far_threshold: FAR threshold for event filtering. For injection analysis only.
 
     Returns:
-        :np.ndarray: samples
-        :np.ndarray: names
+        np.ndarray: samples
+        np.ndarray: names
     '''
     folder      = Path(path).resolve()
     event_files = [Path(folder,f) for f in folder.glob('[!.]*')] # Ignores hidden files
@@ -169,7 +177,7 @@ def load_data(path, seed = False, par = None, n_samples = -1, h = 0.674, om = 0.
             if par is not None:
                 warnings.warn("Par names (or volume keyword) are ignored for .txt/.dat files")
             if n_samples > -1:
-                samples = np.atleast_1d(np.genfromtxt(event))
+                samples = np.atleast_1d(np.loadtxt(event))
                 s = int(min([n_samples, len(samples)]))
                 samples_subset = samples[rdstate.choice(np.arange(len(samples)), size = s, replace = False)]
                 if len(np.shape(samples_subset)) == 1:
@@ -177,7 +185,7 @@ def load_data(path, seed = False, par = None, n_samples = -1, h = 0.674, om = 0.
                 events.append(samples_subset)
                     
             else:
-                samples = np.atleast_1d(np.genfromtxt(event))
+                samples = np.atleast_1d(np.loadtxt(event))
                 if len(np.shape(samples)) == 1:
                     samples = np.atleast_2d(samples).T
                 events.append(samples)
@@ -224,16 +232,16 @@ def _unpack_gw_posterior(event, par, cosmology, rdstate, n_samples = -1, wavefor
         * IMRPhenomPv3HM
     
     Arguments:
-        :str event:            file to read
-        :str par:              parameter to extract
-        :tuple cosmology:      cosmological parameters (h, om, ol)
-        :int n_samples:        number of samples for (random) downsampling. Default -1: all samples
-        :str waveform:         waveform family to be used ('combined', 'imr', 'seob')
-        :double snr_threhsold: SNR threshold for event filtering. For injection analysis only.
-        :double far_threshold: FAR threshold for event filtering. For injection analysis only.
+        str event:            file to read
+        str par:              parameter to extract
+        tuple cosmology:      cosmological parameters (h, om, ol)
+        int n_samples:        number of samples for (random) downsampling. Default -1: all samples
+        str waveform:         waveform family to be used ('combined', 'imr', 'seob')
+        double snr_threhsold: SNR threshold for event filtering. For injection analysis only.
+        double far_threshold: FAR threshold for event filtering. For injection analysis only.
     
     Returns:
-        :np.ndarray:    samples
+        np.ndarray:    samples
     '''
     h, om, ol = cosmology
     omega = CosmologicalParameters(h, om, ol, -1, 0)
@@ -254,6 +262,7 @@ def _unpack_gw_posterior(event, par, cosmology, rdstate, n_samples = -1, wavefor
     with h5py.File(Path(event), 'r') as f:
         samples     = []
         loaded_pars = []
+        flag_filter = False
         try:
             try:
                 # LVK R&P mock data challenge
@@ -334,27 +343,23 @@ def _unpack_gw_posterior(event, par, cosmology, rdstate, n_samples = -1, wavefor
                                         data = f['SEOBNRv4T_surrogate_LS']['posterior_samples']
                 
             for name, lab in zip(GW_par.keys(), GW_par.values()):
-                flag_filter = False
                 if name in par:
                     if name == 'snr':
                         try:
-                            if MDC_flag:
-                                snr = np.array(data['snr'])
-                                samples.append(data['snr'])
-                            elif waveform != 'combined':
+                            if MDC_flag or waveform != 'combined':
                                 snr = np.array(data[lab])
                                 samples.append(data[lab])
                             if snr_threshold is not None:
                                 flag_filter = True
                         except:
                             warnings.warn("SNR filter is not available with this dataset.")
-                    if name == 'far' and far_threshold is not None:
+                    elif name == 'far' and far_threshold is not None:
                         try:
                             flag_filter = True
                             far = np.array(data[lab])
                         except:
                             warnings.warn("FAR filter is not available with this dataset.")
-                    if name == 's1':
+                    elif name == 's1':
                         try:
                             samples.append(data[lab])
                         except:
@@ -436,6 +441,7 @@ def _unpack_gw_posterior(event, par, cosmology, rdstate, n_samples = -1, wavefor
             ss['mc']      = (ss['m1']*ss['m2'])**(3./5.)/(ss['m1']+ss['m2'])**(1./5.)
             ss['mt']      = ss['m1']+ss['m2']
             ss['q']       = ss['m2']/ss['m1']
+            ss['eta']     = ss['m1']*ss['m2']/(ss['m1']+ss['m2'])**2
             ss['s1z']     = ss['s1']*ss['cos_tilt_1']
             ss['s2z']     = ss['s2']*ss['cos_tilt_2']
             ss['chi_eff'] = (ss['s1z'] + ss['q']*ss['s2z'])/(1+ss['q'])
@@ -458,7 +464,6 @@ def _unpack_gw_posterior(event, par, cosmology, rdstate, n_samples = -1, wavefor
                         samples.append(samples_loaded[np.where(loaded_pars == pi)[0]].flatten())
                 samples = np.array(samples)
                 samples = samples.T
-
             if n_samples > -1:
                 s = int(min([n_samples, len(samples)]))
                 return samples[rdstate.choice(np.arange(len(samples)), size = s, replace = False)]
@@ -470,10 +475,10 @@ def save_density(draws, folder = '.', name = 'density', ext = 'pkl'):
     Exports a list of figaro.mixture instances to file
 
     Arguments:
-        :list draws:         list of mixtures to be saved
-        :str or Path folder: folder in which the output file will be saved
-        :str name:           name to be given to output file
-        :str ext:            file extension (pkl or json)
+        list draws:         list of mixtures to be saved
+        str or Path folder: folder in which the output file will be saved
+        str name:           name to be given to output file
+        str ext:            file extension (pkl or json)
     """
     if ext == 'pkl':
         with open(Path(folder, name+'.pkl'), 'wb') as f:
@@ -503,10 +508,10 @@ def load_density(path):
     If the requested file extension (pkl or json) is not available, it tries loading the other.
 
     Arguments:
-        :str or Path path: path with draws (file or folder)
+        str or Path path: path with draws (file or folder)
 
     Returns
-        :list: figaro.mixture object instances
+        list: figaro.mixture object instances
     """
     path = Path(path)
     if path.is_file():
@@ -520,10 +525,10 @@ def _load_density_file(file):
     If the requested file extension (pkl or json) is not available, it tries loading the other.
 
     Arguments:
-        :str or Path file: file with draws
+        str or Path file: file with draws
 
     Returns
-        :list: figaro.mixture object instances
+        list: figaro.mixture object instances
     """
     file = Path(file)
     ext  = file.suffix
@@ -551,10 +556,10 @@ def _load_pkl(file):
     Loads a list of figaro.mixture instances from pkl file
 
     Arguments:
-        :str or Path file: file with draws
+        str or Path file: file with draws
 
     Returns
-        :list: figaro.mixture object instances
+        list: figaro.mixture object instances
     """
     with open(file, 'rb') as f:
         draws = dill.load(f)
@@ -565,10 +570,10 @@ def _load_json(file):
     Loads a list of figaro.mixture instances from json file
 
     Arguments:
-        :str or Path file: file with draws
+        str or Path file: file with draws
 
     Returns
-        :list: figaro.mixture object instances
+        list: figaro.mixture object instances
     """
     with open(Path(file), 'r') as fjson:
         dictjson = json.loads(json.load(fjson))
