@@ -24,11 +24,12 @@ class worker:
                        ext         = 'pkl',
                        se_sigma    = None,
                        hier_sigma  = None,
-                       all_samples = None,
+                       scale       = None,
+                       events      = None,
                        label       = None,
                        unit        = None,
                        save_se     = True,
-                       MC_draws    = 2000,
+                       MC_draws    = None,
                        probit      = True,
                        ):
         self.dim                  = bounds.shape[0]
@@ -38,14 +39,17 @@ class worker:
                                            MC_draws   = MC_draws,
                                            probit     = probit,
                                            prior_pars = get_priors(self.bounds,
-                                                                   samples      = all_samples,
+                                                                   samples      = events,
                                                                    std          = hier_sigma,
+                                                                   scale        = scale,
                                                                    probit       = probit,
                                                                    hierarchical = True,
                                                                    )
                                             )
         self.out_folder_plots = out_folder_plots
         self.out_folder_draws = out_folder_draws
+        self.se_sigma         = se_sigma
+        self.scale            = scale
         self.save_se          = save_se
         self.label            = label
         self.unit             = unit
@@ -60,7 +64,7 @@ class worker:
         ev = np.copy(samples)
         ev.setflags(write = True)
         # Actual inference
-        prior_pars = get_priors(self.bounds, samples = ev, probit = self.probit, std = self.se_sigma, hierarchical = False)
+        prior_pars = get_priors(self.bounds, samples = ev, probit = self.probit, std = self.se_sigma, scale = self.scale, hierarchical = False)
         self.mixture.initialise(prior_pars = prior_pars)
         draws      = [self.mixture.density_from_samples(ev) for _ in range(n_draws)]
         # Plots
@@ -127,8 +131,9 @@ def main():
     parser.add_option("-e", "--events", dest = "run_events", action = 'store_false', help = "Skip single-event analysis", default = True)
     parser.add_option("--se_sigma_prior", dest = "se_sigma_prior", type = "string", help = "Expected standard deviation (prior) for single-event inference - single value or n-dim values. If None, it is estimated from samples", default = None)
     parser.add_option("--sigma_prior", dest = "sigma_prior", type = "string", help = "Expected standard deviation (prior) for hierarchical inference - single value or n-dim values. If None, it is estimated from samples", default = None)
+    parser.add_option("--fraction", dest = "scale", type = "float", help = "Fraction of samples standard deviation for sigma prior. Overrided by sigma_prior.", default = None)
     parser.add_option("--n_parallel", dest = "n_parallel", type = "int", help = "Number of parallel threads", default = 4)
-    parser.add_option("--mc_draws", dest = "mc_draws", type = "int", help = "Number of draws for assignment MC integral", default = 2000)
+    parser.add_option("--mc_draws", dest = "mc_draws", type = "int", help = "Number of draws for assignment MC integral", default = None)
     parser.add_option("--snr_threshold", dest = "snr_threshold", type = "float", help = "SNR threshold for simulated GW datasets", default = None)
     parser.add_option("--far_threshold", dest = "far_threshold", type = "float", help = "FAR threshold for simulated GW datasets", default = None)
     parser.add_option("--no_probit", dest = "probit", action = 'store_false', help = "Disable probit transformation", default = True)
@@ -222,7 +227,6 @@ def main():
         print("Ignoring points outside bounds.")
         for i, ev in enumerate(events):
             events[i] = ev[np.where((np.prod(options.bounds[:,0] < ev, axis = 1) & np.prod(ev < options.bounds[:,1], axis = 1)))]
-        all_samples = np.atleast_2d(np.concatenate(events))
     else:
         # Check if all samples are within bounds
         all_samples = np.atleast_2d(np.concatenate(events))
@@ -253,7 +257,8 @@ def main():
                                         ext              = options.ext,
                                         se_sigma         = options.se_sigma_prior,
                                         hier_sigma       = options.sigma_prior,
-                                        all_samples      = all_samples,
+                                        scale            = options.scale,
+                                        events           = events,
                                         label            = symbols,
                                         unit             = units,
                                         save_se          = options.save_single_event,
