@@ -103,24 +103,24 @@ def plot_median_cr(draws, injected = None, samples = None, selfunc = None, bound
     Plot the recovered 1D distribution along with the injected distribution and samples from the true distribution (both if available).
     
     Arguments:
-        :iterable draws:                  container for mixture instances
-        :callable or np.ndarray injected: injected distribution (if available)
-        :np.ndarray samples:              samples from the true distribution (if available)
-        :callable or np.ndarray selfunc:  selection function (if available)
-        :iterable bounds:                 bounds for the recovered distribution. If None, bounds from mixture instances are used.
-        :str or Path out_folder:          output folder
-        :str name:                        name to be given to outputs
-        :int n_pts:                       number of points for linspace
-        :str label:                       LaTeX-style quantity label, for plotting purposes
-        :str unit:                        LaTeX-style quantity unit, for plotting purposes
-        :bool hierarchical:               hierarchical inference, for plotting purposes
-        :bool save:                       whether to save the plots or not
-        :bool show:                       whether to show the plots during the run or not
-        :bool subfolder:                  whether to save the plots in different subfolders (for multiple events)
-        :float true_value:                true value to infer
-        :str true_value_label:            label to assign to the true value marker
-        :str injected_label:              label to assign to the injected distribution
-        :str median_label:                label to assign to the reconstruction
+        iterable draws:                  container for mixture instances
+        callable or np.ndarray injected: injected distribution (if available)
+        np.ndarray samples:              samples from the true distribution (if available)
+        callable or np.ndarray selfunc:  selection function (if available)
+        iterable bounds:                 bounds for the recovered distribution. If None, bounds from mixture instances are used.
+        str or Path out_folder:          output folder
+        str name:                        name to be given to outputs
+        int n_pts:                       number of points for linspace
+        str label:                       LaTeX-style quantity label, for plotting purposes
+        str unit:                        LaTeX-style quantity unit, for plotting purposes
+        bool hierarchical:               hierarchical inference, for plotting purposes
+        bool save:                       whether to save the plots or not
+        bool show:                       whether to show the plots during the run or not
+        bool subfolder:                  whether to save the plots in different subfolders (for multiple events)
+        float true_value:                true value to infer
+        str true_value_label:            label to assign to the true value marker
+        str injected_label:              label to assign to the injected distribution
+        str median_label:                label to assign to the reconstruction
     """
     if median_label is None:
         if hierarchical:
@@ -145,14 +145,16 @@ def plot_median_cr(draws, injected = None, samples = None, selfunc = None, bound
             x_max = bounds[1]
 
     fig, ax = plt.subplots()
+
     # If samples are available, use them as bounds
     if samples is not None:
         ax.hist(samples, bins = int(np.sqrt(len(samples))), histtype = 'step', density = True, label = '$\mathrm{Samples}$', log = True)
-        x_min_l, x_max_l = ax.get_xlim()
-        x_min = np.max((x_min, x_min_l))
-        x_max = np.min((x_max, x_max_l))
-        
-    x    = np.linspace(x_min, x_max, n_pts+2)[1:-1]
+        if bounds is None:
+            x_min_l, x_max_l = ax.get_xlim()
+            x_min = np.max((x_min, x_min_l))
+            x_max = np.min((x_max, x_max_l))
+    xlim = (x_min, x_max)
+    x    = np.linspace(x_min, x_max, n_pts)
     dx   = x[1]-x[0]
     
     probs = np.array([d.pdf(x) for d in draws])
@@ -167,7 +169,6 @@ def plot_median_cr(draws, injected = None, samples = None, selfunc = None, bound
 
     # Samples (if available)
     if samples is not None:
-        xlim = ax.get_xlim()
         ylim = ax.get_ylim()
     else:
         ax.set_yscale('log')
@@ -295,29 +296,28 @@ def plot_median_cr(draws, injected = None, samples = None, selfunc = None, bound
         plt.close()
     
 
-def plot_multidim(draws, samples = None, bounds = None, out_folder = '.', name = 'density', labels = None, units = None, hierarchical = False, show = False, save = True, subfolder = False, n_pts = 200, true_value = None, figsize = 7, levels = [0.5, 0.68, 0.9], scatter_points = False, median_label = None):
+def plot_multidim(draws, samples = None, bounds = None, out_folder = '.', name = 'density', labels = None, units = None, hierarchical = False, show = False, save = True, subfolder = False, n_pts = 200, true_value = None, levels = [0.5, 0.68, 0.9], scatter_points = False, median_label = None):
     """
     Plot the recovered multidimensional distribution along with samples from the true distribution (if available) as corner plot.
     
     Arguments:
-        :iterable draws:         container for mixture instances
-        :int dim:                number of dimensions
-        :np.ndarray samples:     samples from the true distribution (if available)
-        :iterable bounds:        bounds for the recovered distribution. If None, bounds from mixture instances are used.
-        :str or Path out_folder: output folder
-        :str name:               name to be given to outputs
-        :list-of-str labels:     LaTeX-style quantity label, for plotting purposes
-        :list-of-str units:      LaTeX-style quantity unit, for plotting purposes
-        :bool hierarchical:      hierarchical inference, for plotting purposes
-        :bool save:              whether to save the plot or not
-        :bool show:              whether to show the plot during the run or not
-        :bool subfolder:         whether to save in a dedicated subfolder
-        :int n_pts:              number of grid points (same for each dimension)
-        :iterable true_value:    true value to plot
-        :double figsize:         figure size (matplotlib)
-        :iterable levels:        credible levels to plot
-        :bool scatter_points:    scatter samples on 2d plots
-        :str median_label:       label to assign to the reconstruction
+        iterable draws:         container for mixture instances
+        int dim:                number of dimensions
+        np.ndarray samples:     samples from the true distribution (if available)
+        iterable bounds:        bounds for the recovered distribution. If None, bounds from mixture instances are used.
+        str or Path out_folder: output folder
+        str name:               name to be given to outputs
+        list-of-str labels:     LaTeX-style quantity label, for plotting purposes
+        list-of-str units:      LaTeX-style quantity unit, for plotting purposes
+        bool hierarchical:      hierarchical inference, for plotting purposes
+        bool save:              whether to save the plot or not
+        bool show:              whether to show the plot during the run or not
+        bool subfolder:         whether to save in a dedicated subfolder
+        int n_pts:              number of grid points (same for each dimension)
+        iterable true_value:    true value to plot
+        iterable levels:        credible levels to plot
+        bool scatter_points:    scatter samples on 2d plots
+        str median_label:       label to assign to the reconstruction
     """
     
     dim = draws[0].dim
@@ -327,7 +327,7 @@ def plot_multidim(draws, samples = None, bounds = None, out_folder = '.', name =
         else:
             median_label = '\mathrm{DPGMM}'
     if labels is None:
-        labels = ['$x_{0}$'.format(i+1) for i in range(dim)]
+        labels = ['$x_{'+'{0}'.format(i+1)+'}$' for i in range(dim)]
     else:
         labels = ['${0}$'.format(l) for l in labels]
     
@@ -335,7 +335,10 @@ def plot_multidim(draws, samples = None, bounds = None, out_folder = '.', name =
         labels = [l[:-1]+'\ [{0}]$'.format(u) if not u == '' else l for l, u in zip(labels, units)]
     
     levels = np.atleast_1d(levels)
-
+    
+    ext_bounds = False
+    if bounds is not None:
+        ext_bounds = True
     all_bounds = np.atleast_2d([d.bounds for d in draws])
     x_min = np.min(all_bounds, axis = -1).max(axis = 0)
     x_max = np.max(all_bounds, axis = -1).min(axis = 0)
@@ -358,22 +361,22 @@ def plot_multidim(draws, samples = None, bounds = None, out_folder = '.', name =
     bounds = np.array([x_min, x_max]).T
     
     K = dim
-    factor = 2.0          # size of one side of one panel
+    factor = 3.0          # size of one side of one panel
     lbdim = 0.5 * factor  # size of left/bottom margin
     trdim = 0.2 * factor  # size of top/right margin
-    whspace = 0.1         # w/hspace size
+    whspace = 0.2         # w/hspace size
     plotdim = factor * dim + factor * (K - 1.0) * whspace
     dim_plt = lbdim + plotdim + trdim
     
-    fig, axs = plt.subplots(K, K, figsize=(figsize, figsize))
+    fig, axs = plt.subplots(K, K, figsize=(dim_plt, dim_plt))
     # Format the figure.
     lb = lbdim / dim_plt
     tr = (lbdim + plotdim) / dim_plt
     fig.subplots_adjust(left=lb, bottom=lb, right=tr, top=tr, wspace=whspace, hspace=whspace)
     # Samples (if available)
     if samples is not None:
-        bins = [int(np.sqrt(len(samples[:, c]))) for c in range(dim)]
-        corner(samples, color = '#1f77b4', fig = fig, hist_kwargs = {'density': True, 'label':'$\mathrm{Samples}$', 'linewidth':0.7} , plot_density = False, contour_kwargs = {'linewidths':0.3, 'linestyles':'dashed'}, levels = [0.5,0.68,0.9], no_fill_contours = True, bins = bins)
+        bins = np.array([int(np.sqrt(len(samples[:, c]))) for c in range(dim)])
+        corner(samples, color = '#1f77b4', fig = fig, hist_kwargs = {'density': True, 'label':'$\mathrm{Samples}$', 'linewidth':0.7} , plot_density = False, contour_kwargs = {'linewidths':0.3, 'linestyles':'dashed'}, levels = [0.5,0.68,0.9], no_fill_contours = True, hist_bin_factor = bins/20, quiet = True)
         
     # 1D plots (diagonal)
     for column in range(K):
@@ -384,12 +387,12 @@ def plot_multidim(draws, samples = None, bounds = None, out_folder = '.', name =
         marg_draws = marginalise(draws, dims)
         # Credible regions
         lim = bounds[column]
-        if samples is not None:
+        if samples is not None and not ext_bounds:
             lim_l = ax.get_xlim()
             lim[0] = np.max((lim[0], lim_l[0]))
             lim[1] = np.min((lim[1], lim_l[1]))
             
-        x   = np.linspace(lim[0], lim[1], n_pts+2)[1:-1]
+        x   = np.linspace(lim[0], lim[1], n_pts)
         dx  = x[1]-x[0]
         
         probs = np.array([d.pdf(x) for d in marg_draws])
@@ -441,7 +444,7 @@ def plot_multidim(draws, samples = None, bounds = None, out_folder = '.', name =
             
             # Credible regions
             lim = bounds[[row, column]]
-            if samples is not None:
+            if samples is not None and not ext_bounds:
                 lim_l = np.array([ax.get_ylim(), ax.get_xlim()])
                 for i in range(2):
                     lim[i,0] = np.max((lim[i,0], lim_l[i,0]))
@@ -449,8 +452,8 @@ def plot_multidim(draws, samples = None, bounds = None, out_folder = '.', name =
                 
             grid, dgrid = recursive_grid(lim[::-1], np.ones(2, dtype = int)*int(n_pts))
             
-            x = np.linspace(lim[0,0], lim[0,1], n_pts+2)[1:-1]
-            y = np.linspace(lim[1,0], lim[1,1], n_pts+2)[1:-1]
+            x = np.linspace(lim[0,0], lim[0,1], n_pts)
+            y = np.linspace(lim[1,0], lim[1,1], n_pts)
             
             dd = np.array([d.pdf(grid) for d in marg_draws])
             median = np.percentile(dd, 50, axis = 0)
@@ -487,10 +490,13 @@ def plot_multidim(draws, samples = None, bounds = None, out_folder = '.', name =
             if column == 0:
                 ax.set_ylabel(labels[row])
                 [l.set_rotation(45) for l in ax.get_yticklabels()]
+            else:
+                ax.set_yticklabels([])
             if row == K - 1:
                 [l.set_rotation(45) for l in ax.get_xticklabels()]
                 ax.set_xlabel(labels[column])
-
+            else:
+                ax.set_xticklabels([])
     fig.axes[K-1].legend(*fig.axes[0].get_legend_handles_labels(), loc = 'center')
     fig.align_labels()
     
@@ -516,23 +522,23 @@ def plot_1d_dist(x, draws, injected = None, samples = None, out_folder = '.', na
     For FIGARO mixture instances, please use plot_median_cr.
 
     Arguments:
-        :iterable x:                      values at which realisations are evaluated
-        :iterable draws:                  container for realisations
-        :callable or np.ndarray injected: injected distribution (if available)
-        :np.ndarray samples:              samples from the true distribution (if available)
-        :str or Path out_folder:          output folder
-        :str name:                        name to be given to outputs
-        :str label:                       LaTeX-style quantity label, for plotting purposes
-        :str unit:                        LaTeX-style quantity unit, for plotting purposes
-        :bool save:                       whether to save the plots or not
-        :bool show:                       whether to show the plots during the run or not
-        :bool subfolder:                  whether to save the plots in different subfolders (for multiple events)
-        :float true_value:                true value to infer
-        :str true_value_label:            label to assign to the true value marker
-        :str injected_label:              label to assign to the injected distribution
-        :str median_label:                label to assign to the median distribution
-        :bool logx:                       x log scale
-        :bool logy:                       y log scale
+        iterable x:                      values at which realisations are evaluated
+        iterable draws:                  container for realisations
+        callable or np.ndarray injected: injected distribution (if available)
+        np.ndarray samples:              samples from the true distribution (if available)
+        str or Path out_folder:          output folder
+        str name:                        name to be given to outputs
+        str label:                       LaTeX-style quantity label, for plotting purposes
+        str unit:                        LaTeX-style quantity unit, for plotting purposes
+        bool save:                       whether to save the plots or not
+        bool show:                       whether to show the plots during the run or not
+        bool subfolder:                  whether to save the plots in different subfolders (for multiple events)
+        float true_value:                true value to infer
+        str true_value_label:            label to assign to the true value marker
+        str injected_label:              label to assign to the injected distribution
+        str median_label:                label to assign to the median distribution
+        bool logx:                       x log scale
+        bool logy:                       y log scale
     """
     
     if not np.shape(x)[0] == np.shape(draws)[-1]:
@@ -626,12 +632,12 @@ def plot_n_clusters_alpha(n_cl, alpha, out_folder = '.', name = 'event', show =
     Plot the number of clusters and the concentration parameter as functions of the number of samples.
     
     Arguments:
-        :np.ndarray n_cl:        number of active clusters
-        :np.ndarray alpha:       concentration parameter
-        :str or Path out_folder: output folder
-        :str name:               name to be given to outputs
-        :bool save:              whether to save the plot or not
-        :bool show:              whether to show the plot during the run or not
+        np.ndarray n_cl:        number of active clusters
+        np.ndarray alpha:       concentration parameter
+        str or Path out_folder: output folder
+        str name:               name to be given to outputs
+        bool save:              whether to save the plot or not
+        bool show:              whether to show the plot during the run or not
     """
     fig, ax = plt.subplots()
     ax1 = ax.twinx()
@@ -651,18 +657,18 @@ def pp_plot_cdf(draws, injection, n_points = 1000, out_folder = '.', name = 'eve
     Make pp-plot comparing draws cdfs and injection cdf
     
     Arguments:
-        :iterable draws:         container of mixture instances
-        :callable injection:     injected density
-        :int n_points:           number of points for linspace
-        :str or Path out_folder: output folder
-        :str name:               name to be given to outputs
-        :bool save:              whether to save the plot or not
-        :bool show:              whether to show the plot during the run or not
+        iterable draws:         container of mixture instances
+        callable injection:     injected density
+        int n_points:           number of points for linspace
+        str or Path out_folder: output folder
+        str name:               name to be given to outputs
+        bool save:              whether to save the plot or not
+        bool show:              whether to show the plot during the run or not
     """
     all_bounds = np.atleast_2d([d.bounds[0] for d in draws])
     x_min = np.max(all_bounds[:,0])
     x_max = np.min(all_bounds[:,1])
-    x = np.linspace(x_min, x_max, n_points+2)[1:-1]
+    x = np.linspace(x_min, x_max, n_points)
     
     functions     = np.array([mix(x) for mix in draws])
     median        = np.percentile(functions, 50, axis = 0)
@@ -690,12 +696,12 @@ def pp_plot_levels(CR_levels, median_CR = None, out_folder = '.', name = 'MDC',
     Make pp-plot.
     
     Arguments:
-        :iterable CR:            2D array with credible levels for each event
-        :iterable median_CR:     credible levels of medians
-        :str or Path out_folder: output folder
-        :str name:               name to be given to outputs
-        :bool save:              whether to save the plot or not
-        :bool show:              whether to show the plot during the run or not
+        iterable CR:            2D array with credible levels for each event
+        iterable median_CR:     credible levels of medians
+        str or Path out_folder: output folder
+        str name:               name to be given to outputs
+        bool save:              whether to save the plot or not
+        bool show:              whether to show the plot during the run or not
     """
     CR_levels = np.atleast_1d(CR_levels)
     if len(CR_levels.shape) > 1:
